/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2000-2003 Justin Kirby
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 */
#ifndef OAPR_IO_HPP
#define OAPR_IO_HPP

#include <string>
#include "apr.h"
#include "oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_pool.hpp"
#include "oapr/oapr_pool_mgr.hpp"

FILE_VERSION("$Revision: 1.6.2.2 $",oapr_io_hpp);

namespace oapr
{
  /**
   * Base class for all io. 
   * This is used for files, sockets, pipes, any input/output
   */
  class LIBOAPR_API basic_io
  {
  public:
    typedef pool_mgr::pool_t pool_t;
    typedef pool_mgr::pool_ptr pool_ptr;
  public:
#ifdef WIN32
	  basic_io(){}
#endif
    basic_io(const pool::pool_ptr& p)
      :m_pool(p)
    {
    }
    basic_io(const basic_io& i)
      :	m_pool(i.m_pool)
    {
    }
    virtual ~basic_io()
    {
    }
    virtual pool::pool_ptr gpool()const{return m_pool;}
    /**
     * closes the resource, but does not free resources.
     */
    virtual apr_status_t close()=0;

    /**
     * End of file check
     * @return returns true if eof and false otherwise
     */
    virtual bool eof()=0;

    /**
     * Checks to see if the device is ready for io.
     * @return true if ready, false otherwise
     */
    virtual bool is_open()const=0;

    /**
     * Writes the buff content out to the device
     * @param buff buffer to write
     * @param toWrite number of bytes to write
     * @param full if true, forces a complete write of the buffer if possible
     * @return number of bytes written.
     */
    virtual apr_size_t write(const void* buff,
			     apr_size_t toWrite,
			     bool full=false)=0;

    /**
     * Reads contents of the devices into the buffer
     * @param buff buffer to put contents 
     * @param toRead number of bytes to read
     * @param full force fill of buffer
     * @return number of bytes actually read
     */
    virtual apr_size_t read( void* buff,
			    apr_size_t toRead,
			    bool full=false)const=0;

    virtual apr_status_t flush()=0;
  protected:
    pool::pool_ptr m_pool;
  };
};

#endif
