/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#ifndef OAPR_MUTEX_HPP
#define OAPR_MUTEX_HPP

#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_pool.hpp"
#include "apr_thread_mutex.h"
#include <assert.h>
FILE_VERSION("$Revision: 1.4.2.3 $",oapr_mutex_hpp);

namespace oapr
{
  class condition;
  class LIBOAPR_API mutex
  {
  private:
    friend class mlock;
    friend class condition;
  public:

    mutex(const pool::pool_ptr& p);
    ~mutex();
#ifdef WIN32
  public:
#else
  private:
#endif
    mutex() {}
  private:
    apr_thread_mutex_t *handle() { return m_mutex; }
    apr_thread_mutex_t *m_mutex;
    pool::pool_ptr m_pool;
  };

  /**
   * An object protected by a rw lock.
   */
  class shared_object
  {
    friend class mlock;
    private:
      mutex _lock;
  };

  /**
   * EXPERIMENTAL:
   * A thread-safe shared object wrapped by a mutex lock.
   */
  template<class T> class shared: public T
  {
    friend class mlock;
    private:
      mutex _lock;
  };

  /**
   * Scoped mutex lock. Holds a write lock on a mutex, mutexobj or shared<> object.
   */
  class LIBOAPR_API mlock
  {
    public:
      explicit mlock(mutex& target): m_lock(target.m_mutex) { apr_thread_mutex_lock(m_lock); }
      explicit mlock(shared_object& target): m_lock(target._lock.m_mutex) { apr_thread_mutex_lock(m_lock); }
      /**
       * Cancel the mutex lock before end of scope.
       */
      void cancel() { apr_thread_mutex_unlock(m_lock); m_lock = NULL; }
      void reacquire(mutex& target) { if(m_lock) cancel(); m_lock = target.m_mutex; apr_thread_mutex_lock(m_lock); }
      ~mlock() { if(m_lock) apr_thread_mutex_unlock(m_lock); }
    private:
      apr_thread_mutex_t *m_lock;
  };


};

#endif
