/* ====================================================================
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2003 Matt Carter. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Apache" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

#include "oapr/oapr.hpp"
#include "oapr/oapr_buildconfig.hpp"
#include "oapr/oapr_process.hpp"

FILE_VERSION("$Revision: 1.4 $",oapr_process_cpp);

namespace oapr {

child_process::child_process(pool& ap)
  : m_pool(&ap),
    m_assigned(false)
{
}

child_process::child_process(pool& ap, const std::string& progname, std::vector<std::string>& args, const std::vector<std::string>& env)
  : m_pool(&ap),
    m_assigned(false)
{
}

child_process::~child_process()
{
  if(m_assigned)
  {
    if(!done()) kill(SIGTERM);
    waitfor();
  };
}

void child_process::create_proc(const std::string& progname, std::vector<std::string>& args, const std::vector<std::string>& env)
{
  assert(!m_assigned);
  m_progname = progname;
  /* Prepare args */
  /**\todo args, env*/
  /* Create process */
  OAPR_CHECK(apr_proc_create(&m_proc, m_progname.c_str(), NULL, NULL, NULL /*attrs*/, m_pool->getp()));
  m_assigned = true;
}

void child_process::waitfor(int *exitcode, apr_exit_why_e *exitwhy)
{
  assert(m_assigned);
  apr_status_t rc = apr_proc_wait(&m_proc, exitcode, exitwhy, APR_WAIT);
  if(rc == APR_CHILD_DONE || rc == APR_SUCCESS) return;
  throw oapr_exception(rc, "child_process::waitfor()");
}

bool child_process::done(int *exitcode, apr_exit_why_e *exitwhy)
{
  assert(m_assigned);
  apr_status_t rc = apr_proc_wait(&m_proc, exitcode, exitwhy, APR_NOWAIT);
  if(rc == APR_CHILD_NOTDONE) return false;
  if(rc == APR_CHILD_DONE) return true;
  throw oapr_exception(rc, "child_process::done()");
}

apr_status_t this_process::detach(daemon_option_t daemonize)
{
  return 0;
  /**@bug linking issues with this from the apr lib*/
  //return apr_proc_detach(daemonize == background ? 1 : 0);
}

apr_status_t this_process::fork(process &new_process)
{
  return APR_ENOTIMPL;
}

};
